@@ -449,26 +449,39 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   if (lag_size < 1){
     Rf_error("lag must be a non-zero length integer vector");
   }
-  bool has_order = o_size > 0;
-  bool has_rl = rl_size > 0;
+  bool has_order = !Rf_isNull(order);
+  bool has_rl = !Rf_isNull(run_lengths);
   bool recycle_lag = lag_size != 1;
+
+  // When order is NULL we run through x from left to right (as usual)
+  // When run_lengths is NULL we run through x without resetting
+  // To do this properly we use dummy vectors so that
+  // we can statically assign int pointers
+  // While keeping order and run_lengths as NULL (if they are NULL)
+  // This is mainly done this way because lag2_ is recursive and
+  // hence order/run_lengths should remain NULL throughout each recursion
+  // if they are NULL
+
+  SEXP dummy_vec1 = Rf_protect(Rf_allocVector(INTSXP, 0));
+  ++n_protections;
+  SEXP dummy_vec2 = Rf_protect(Rf_allocVector(INTSXP, 0));
+  ++n_protections;
   Rf_protect(lag = Rf_coerceVector(lag, INTSXP));
   ++n_protections;
-  Rf_protect(order = Rf_coerceVector(order, INTSXP));
+  Rf_protect(order = has_order ? Rf_coerceVector(order, INTSXP) : R_NilValue);
   ++n_protections;
-  Rf_protect(run_lengths = Rf_coerceVector(run_lengths, INTSXP));
+  Rf_protect(run_lengths = has_rl ? Rf_coerceVector(run_lengths, INTSXP) : R_NilValue);
   ++n_protections;
   // std::variant<int*, double*> p_o;
   // typedef std::conditional<true, int*, double*>::type p_o;
-  int *p_o = INTEGER(order);
-  int *p_rl = INTEGER(run_lengths);
+  int *p_o = INTEGER(has_order ? order : dummy_vec1);
+  int *p_rl = INTEGER(has_rl ? run_lengths : dummy_vec2);
   int *p_lag = INTEGER(lag);
   int rl; // Run-length
   int run_start = 0; // Start index of current run
   int run_end = 0; // End index of current run
-  int oi;
-  int lagi;
-  int k;
+  int oi; // Indices (specified by order vector) to lag
+  int k; // Lag
   int lag1 = p_lag[0];
   // Manually set run rl size to 1 if run_lengths = NULL (checked prior)
   if (!has_rl) rl_size = 1;
@@ -478,11 +491,13 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   if (!Rf_isNull(Rf_getAttrib(x, R_NamesSymbol))){             \
     SEXP old_names = Rf_protect(Rf_getAttrib(x, R_NamesSymbol));\
     ++n_protections;                                           \
-    SEXP new_names = Rf_protect(cpp_lag2(old_names, lag, order, run_lengths, fill, recursive));\
+    SEXP new_names = Rf_protect(cpp_lag2(old_names, lag, order, run_lengths, R_NilValue, recursive));\
     ++n_protections;                                           \
     Rf_setAttrib(out, R_NamesSymbol, new_names);               \
   }                                                            \
 
+  // Initialise range of possible order values to fast check
+  // user-supplied order values
   unsigned int o_rng = o_size - 1;
   SEXP out;
   switch(TYPEOF(x)){
@@ -494,7 +509,7 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   case INTSXP: {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     int *p_x = INTEGER(x);
     int fill_value = NA_INTEGER;
@@ -519,10 +534,10 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
       // int trick to calculate inclusive bounded between(x, lo, hi)
-      unsigned int rng = (run_end - 1) - run_start;
+      // unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -534,13 +549,18 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
         }
         // Costly to use % if we don't need to
         k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        lagi = j - k;
-        p_out[oi] = unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_value;
+        if (k >= 0){
+          p_out[oi] = (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
+        } else {
+          p_out[oi] = (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
+        }
+        // lagi = (k != NA_INTEGER) ? j - k : rng + 1;
+        // p_out[oi] = unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_value;
       }
     }
     if (run_end != size){
       Rf_unprotect(n_protections);
-      Rf_error("sum(run_lengths) must be equal to length(x)");
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
     LAG_R_NAMES;
     break;
@@ -548,7 +568,7 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   case REALSXP: {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     double *p_x = REAL(x);
     double fill_value = NA_REAL;
@@ -572,9 +592,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -585,13 +604,16 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
           oi = j;
         }
         k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        lagi = j - k;
-        p_out[oi] = unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_value;
+        if (k >= 0){
+          p_out[oi] = (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
+        } else {
+          p_out[oi] = (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value;
+        }
       }
     }
     if (run_end != size){
       Rf_unprotect(n_protections);
-      Rf_error("sum(run_lengths) must be equal to length(x)");
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
     LAG_R_NAMES;
     break;
@@ -599,10 +621,10 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   case STRSXP: {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     SEXP *p_x = STRING_PTR(x);
-    SEXP fill_char = Rf_protect(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING);
+    SEXP fill_value = Rf_protect(fill_size >= 1 ? Rf_asChar(fill) : NA_STRING);
     ++n_protections;
     out = Rf_protect(Rf_duplicate(x));
     ++n_protections;
@@ -620,9 +642,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -633,13 +654,16 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
           oi = j;
         }
         k = recycle_lag ? p_lag[j % lag_size] : p_lag[0];
-        lagi = j - k;
-        SET_STRING_ELT(out, oi, unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_char);
+        if (k >= 0){
+          SET_STRING_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        } else {
+          SET_STRING_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        }
       }
     }
     if (run_end != size){
       Rf_unprotect(n_protections);
-      Rf_error("sum(run_lengths) must be equal to length(x)");
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
     LAG_R_NAMES;
     break;
@@ -647,7 +671,7 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   case CPLXSXP: {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     Rcomplex *p_x = COMPLEX(x);
     SEXP fill_sexp = Rf_protect(Rf_allocVector(CPLXSXP, 1));
@@ -672,9 +696,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -685,13 +708,16 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
           oi = j;
         }
         k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        lagi = j - k;
-        SET_COMPLEX_ELT(out, oi, unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_value);
+        if (k >= 0){
+          SET_COMPLEX_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        } else {
+          SET_COMPLEX_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        }
       }
     }
     if (run_end != size){
       Rf_unprotect(n_protections);
-      Rf_error("sum(run_lengths) must be equal to length(x)");
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
     LAG_R_NAMES;
     break;
@@ -699,11 +725,11 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   case RAWSXP: {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     Rbyte *p_x = RAW(x);
     SEXP raw_sexp = Rf_protect(Rf_coerceVector(fill, RAWSXP));
-    Rbyte fill_raw = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0];
+    Rbyte fill_value = fill_size == 0 ? RAW(Rf_ScalarRaw(0))[0] : RAW(raw_sexp)[0];
     ++n_protections;
     out = Rf_protect(Rf_duplicate(x));
     ++n_protections;
@@ -721,9 +747,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -734,10 +759,17 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
           oi = j;
         }
         k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        lagi = j - k;
-        SET_RAW_ELT(out, oi, unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : fill_raw);
+        if (k >= 0){
+          SET_RAW_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        } else {
+          SET_RAW_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        }
       }
     }
+    if (run_end != size){
+      Rf_unprotect(n_protections);
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
+    }
     LAG_R_NAMES;
     break;
   }
@@ -754,14 +786,13 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
   } else {
     int size = Rf_length(x);
     if (has_order && (size != o_size)){
-      Rf_error("length(order) must equal length(x)");
+      Rf_error("length(order) must equal length(x) (%d)", size);
     }
     const SEXP *p_x = VECTOR_PTR_RO(x);
-    SEXP fill_value = Rf_protect(Rf_coerceVector(fill_size >= 1 ? fill : R_NilValue, VECSXP));
+    SEXP fill_value = Rf_protect(VECTOR_ELT(Rf_coerceVector(fill_size >= 1 ? fill : R_NilValue, VECSXP), 0));
     ++n_protections;
     out = Rf_protect(Rf_allocVector(VECSXP, size));
     ++n_protections;
-    const SEXP *p_fill = VECTOR_PTR_RO(fill_value);
     for (int i = 0; i != rl_size; ++i){
       run_start = run_end; // Start at the end of the previous run
       rl = has_rl ? p_rl[i] : size; // Current run-length
@@ -776,9 +807,8 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
       // If the cumulative run-length exceeds length(x) we stop
       if (run_end > size){
         Rf_unprotect(n_protections);
-        Rf_error("sum(run_lengths) must be equal to length(x)");
+        Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
       }
-      unsigned int rng = (run_end - 1) - run_start;
       // Loop starting from the end of the previous run-length
       for (int j = run_start; j != run_end; ++j){
         // Check that order value is valid
@@ -789,13 +819,16 @@ SEXP cpp_lag2(SEXP x, SEXP lag, SEXP order, SEXP run_lengths, SEXP fill, bool re
           oi = j;
         }
         k = recycle_lag ? p_lag[j % lag_size] : lag1;
-        lagi = j - k;
-        SET_VECTOR_ELT(out, oi, unsigned(lagi - run_start) <= rng ? p_x[has_order ? p_o[lagi] - 1 : lagi] : p_fill[0]);
+        if (k >= 0){
+          SET_VECTOR_ELT(out, oi, (j - run_start) >= k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        } else {
+          SET_VECTOR_ELT(out, oi, (j - run_end) < k ? p_x[has_order ? p_o[j - k] - 1 : j - k] : fill_value);
+        }
       }
     }
     if (run_end != size){
       Rf_unprotect(n_protections);
-      Rf_error("sum(run_lengths) must be equal to length(x)");
+      Rf_error("sum(run_lengths) must be equal to length(x) (%d)", size);
     }
     LAG_R_NAMES;
   }
